var AjaxCacher = function(jQueryInstance) {

    const AJAX_STATE_NONE = undefined;
    const AJAX_STATE_PENDING = 1;
    const AJAX_STATE_DONE = 2;

    var ajaxState = new Array();
    var cachedData = new Array();
    var pendingCallbacks = new Array();

    var jQuery = $;
    if (jQueryInstance != null) {
        jQuery = jQueryInstance;
    }

    function ajax(data, completedCallback) {
        if (data.cacheId != undefined) {
            if (ajaxState[data.cacheId] == AJAX_STATE_DONE) {
                completedCallback(cachedData[data.cacheId]);
                return;
            } else if (ajaxState[data.cacheId] == AJAX_STATE_PENDING) {
                addPendingCallback(data.cacheId, completedCallback);
                return;
            } else if (ajaxState[data.cacheId] == AJAX_STATE_NONE) {
                ajaxState[data.cacheId] = AJAX_STATE_PENDING;
                addPendingCallback(data.cacheId, completedCallback);
                makeAjaxCall(data.cacheId, data);
            }
        } else {
            jQuery.ajax(data).completed(function(data) {
                if (completedCallback != null) {
                    completedCallback(data);
                }
            });
        }
    }

    function addPendingCallback(cacheId, callback) {
        if (pendingCallbacks[cacheId] == null) {
            pendingCallbacks[cacheId] = new Array();
        }
        pendingCallbacks[cacheId].push(callback);
    }

    function callPendingCallbacks(cacheId) {
        var callbacks = pendingCallbacks[cacheId];
        for (var i = 0; i < callbacks.length; i++) {
            callbacks[i](cachedData[cacheId]);
        }
        pendingCallbacks[cacheId] = undefined;
    }

    function makeAjaxCall(cacheId, ajaxData) {
        jQuery.ajax(ajaxData).always(function(data) {
            cachedData[cacheId] = data;
            ajaxState[cacheId] = AJAX_STATE_DONE;
            callPendingCallbacks(cacheId);
        });
    }

    return {
        ajax : ajax
    }
};
