AJS.$(document).ready(function() {
	var SECONDS_IN_DAY = 24 * 60 * 60;

	var timeline=new vis.Timeline(AJS.$("#timeline")[0]);
	var timelineData={
		options: null,
		groups: null,
		items: null
	};

	var periodType = {
		DAILY: 'DAILY',
		HALF_DAILY: 'HALF_DAILY',
		HOURLY:'HOURLY'
	};

	var halfDailyType = {
		AM: 'AM',
		PM: 'PM'
	};

	timeline.on('select', function (properties) {
		AJS.$("[instanceId]").removeClass("vis-selected");
		for (var i=0; i<properties.items.length; i++) {
			var id=properties.items[i];
			if (typeof id === 'string' && id.includes("used-"))
				id=id.split("-")[1];
			AJS.$("[instanceId="+id+"]").addClass("vis-selected");
		}
	});
	
	AJS.$("#data").hide();
	
	AJS.$(".userSelect").auiSelect2({placeholder: AJS.I18n.getText("vacationmanager.action.statistics.user.select.user.placeholder")});
	AJS.$("#startDate").datePicker({'overrideBrowserDefault': true});
	AJS.$("#endDate").datePicker({'overrideBrowserDefault': true});
	
	AJS.$("#startDate").val(parseDate(new Date()));
	AJS.$("#endDate").val(parseDate(new Date(new Date().getFullYear(), 11, 31)));
	
	function changeTableData(user, start, end) {
		AJS.$("#loading-overlay").show();
		jQuery.ajax({
			type : "GET",
			url : AJS.contextPath() + "/rest/vacation-manager/1.0/statistics/user/" + user
			+ "?startDate=" + VM$.formatDateIso(start) + "&endDate=" + VM$.formatDateIso(end),

			async : true
		}).done(function(data) {
			AJS.$("#loading-overlay").hide();
			populateData(data, start, end);
		});
	}
	
	function populateData(data, start, end) {
		populateInfoHeader(data, start, end);
		clearTimeline();
		
		AJS.$("[vacationKind]").each(function() {
			var obj=AJS.$(this);
			var chart=obj.find("[name=chart]");
			var table=obj.find("[name=table]");
			var vacationKindId=obj.attr("vacationKind");
			var userData=data.vacationTypeInstances[vacationKindId];

			populateTableData(userData, table);
			populateChartData(userData, chart);
			populateTimelineData(userData, data.freeDays);
		});
		renderTimeline(new Date(start), new Date(end));
	}
	
	function populateInfoHeader(data, start, end) {
		var label = AJS.I18n.getText("vacationmanager.action.statistics.user.displaying.for.label",
			data.user.name, data.user.displayName, parseDate(start), parseDate(end));
		AJS.$("#dataInfo").text(label);
	}
	
	function clearTimeline() {
		timelineData.items = new vis.DataSet();
		timelineData.groups = new vis.DataSet();
	}
	
	function populateTimelineData(userData, freeDays) {
		if (!userData) {
			return;
		}

        var timelineGroup = getTimelineGroupFor(userData.vacationType);
        timelineData.groups.add(timelineGroup);

		for (var inst = 0; inst < userData.instances.length; inst++) {
			var instanceFromRest = userData.instances[inst];
			var documents = instanceFromRest.vacationDocuments;

			for (var doc = 0; doc < documents.length; doc++) {
				var document = documents[doc];
				var dates = getTimelineStartAndEndDateTimesForPeriod(document);
				timelineData.items.add({
					id: 'used-' + instanceFromRest.id + '-' + document.id,
					group: timelineGroup.id,
					content: vmUtils.formatDaysAndTime(document.calculatedDays, document.calculatedMinutes, {shortLabels: true}),
                    start: dates.startDate,
                    end: dates.endDate,
                    className: 'vacman-vis-used'
				});
			}
		}

        markFreeDaysForTimelineGroup(freeDays, timelineGroup);
	}

	function getTimelineGroupFor(vacationKind) {
        var vacationKindId = vacationKind.id;
        var vacationKindName = AJS.$('<p/>', {text: vacationKind.name}).html();
        return {
        	id: vacationKindId,
			content: vacationKindName
        };
    }

    function markFreeDaysForTimelineGroup(freeDays, group) {
        freeDays = freeDays || [];

        for (var i = 0; i < freeDays.length; i++) {
            var day = freeDays[i];
            timelineData.items.add({
                group: group.id,
                start: startOfDay(new Date(day)),
                end: endOfDay(new Date(day)),
                className: 'vacman-vis-free',
                type: 'background'
            });
        }
    }

	function renderTimeline(startDate, endDate) {
		timelineData.options = {
			min: startOfDay(startDate),
			max: endOfDay(endDate),
			start: startOfDay(startDate),
			end: endOfDay(endDate),
			margin:{item:{horizontal:0}},
			timeAxis:{scale:"day"},
			type:"range",
			zoomMin: 2 * SECONDS_IN_DAY
		};
		timeline.setOptions(timelineData.options);
		timeline.setGroups(timelineData.groups);
		timeline.setItems(timelineData.items);
	}
	
	function populateTableData(userData, table) {
		if (!userData) {
            table.html(AJS.I18n.getText('vacationmanager.action.statistics.no.data'));
            return;
		} else if (userData.instances.length === 0) {
            table.html(AJS.I18n.getText('vacationmanager.action.statistics.no.definitions'));
            return;
		}

        var notApplicableLabel = AJS.I18n.getText('vacationmanager.text.not.applicable');
		var instances = [];
		var availableDaysSum = 0;
		var carryOverDaysSum = 0;
		var usedDaysSum = 0;
		for (var instanceIndex = 0; instanceIndex < userData.instances.length; instanceIndex++) {
			var instanceFromRest = userData.instances[instanceIndex];
			var startDate = startOfDay(new Date(instanceFromRest.startDate));
			var endDate = endOfDay(new Date(instanceFromRest.endDate));
            var now = new Date();
			var formattedAvailableTime = vmUtils.formatDaysAndTime(instanceFromRest.availableDays, instanceFromRest.availableMinutes);

			var instance = {
				id: instanceFromRest.id,
				startDate: parseDate(startDate),
				endDate: userData.vacationType.hasEndDate ? parseDate(endDate) : notApplicableLabel,
				availableDays: userData.vacationType.hasDayLimit ? instanceFromRest.availableDays : notApplicableLabel,
				availableTime: userData.vacationType.hasDayLimit ? formattedAvailableTime : notApplicableLabel,
				carryOverDays: instanceFromRest.carryOverDays,
				carryOverTime: vmUtils.formatDaysAndTime(instanceFromRest.carryOverDays, instanceFromRest.carryOverMinutes),
				usedTime: vmUtils.formatDaysAndTime(instanceFromRest.usedDays, instanceFromRest.usedMinutes),
				vacationDocuments: instanceFromRest.vacationDocuments,
				current: now >= startDate && now <= endDate
			};

			if (userData.vacationType.hasDayLimit) {
				availableDaysSum += parseInt(instanceFromRest.availableDays);
			}
			carryOverDaysSum += parseInt(instanceFromRest.carryOverDays);
			usedDaysSum += parseInt(instanceFromRest.usedDays);
			instances.push(instance);
		}
		var generatedTable = VacationManager.Templates.UserStatistics.vacationKindInstancesTable({
			instances: instances,
			jqlSearchUrl: AJS.params.baseURL+"/issues/?jql="
		});
		table.html(generatedTable);
	}

	function populateChartData(userData, chart) {
		if (userData) {
			if (userData.instances.length>0) {
				var availableDaysSum=0;
				var carryOverDaysSum=0;
				var usedDaysSum = 0;

				for (var inst=0; inst<userData.instances.length; inst++) {
					var instanceFromRest=userData.instances[inst];
					availableDaysSum += instanceFromRest.availableDays;
					carryOverDaysSum += instanceFromRest.carryOverDays;
					usedDaysSum += instanceFromRest.usedDays;
				}
				drawChart(chart[0], usedDaysSum, availableDaysSum, carryOverDaysSum);
			} else {
				chart.html("");
			}
		} else {
			chart.html("");
		}
	}
	
	var yearMilis = 1000*60*60*24*365;
	
	function checkChangedData() {
		AJS.$("#error").text("");
	
		var user=AJS.$("#userSelect").val();
		var startDate=new Date(AJS.$("#startDate").val());
		var endDate=new Date(AJS.$("#endDate").val());
		
		if (startDate=="Invalid Date") {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.invalid.start.date"));
			return;
		}
		
		if (endDate=="Invalid Date") {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.invalid.end.date"));
			return;
		}
		
		if (startDate>endDate) {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.enddate.before.startdate"));
			return;
		}
		
		if (endDate-startDate > yearMilis * 10) {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.period.too.long"));
			return;
		}
		
		if (user.length<=0) {
			return;
		}
		
		AJS.$("#data").show();
		changeTableData(user, startDate, endDate);
	}

  	function drawChart(div, usedDays, availableDays, leftoverDays) {
  		if (availableDays<0) availableDays=0;
  		if (leftoverDays<0) leftoverDays=0;

        var vacationTimeChart = new VacationDaysPieChart({
            chartSelector: div,
            chartOptions: {
                chartArea: {width: '80%'}
            }
        });

        vacationTimeChart.onLoad(function () {
            vacationTimeChart.draw({
                availableDays: availableDays,
                usedDays: usedDays,
                leftoverDays: leftoverDays
            });
        });
  	}
  	
	AJS.$("[onChangeListened]").change(checkChangedData);
	if (AJS.$("[onInitListened]").length>0) {
		checkChangedData();
	};
	AJS.$('.overlay-spinner').spin();

	function startOfDay(date) {
		var resultDate = new Date(date.getTime());
		resultDate.setHours(0, 0, 0, 0);
		return resultDate;
    }

    function endOfDay(date) {
        var resultDate = new Date(date.getTime());
        resultDate.setHours(23, 59, 59, 999);
        return resultDate;
    }
	
	function parseDate(date) {
		var year=date.getFullYear();
		var month=pad((date.getMonth()+1),2);
		var day=pad((date.getDate()),2);
		return year+"-"+month+"-"+day;
	}
	
	function pad(num, size) {
	    var s = "00000000000000" + num;
	    return s.substr(s.length-size);
	}

    function getTimelineStartAndEndDateTimesForPeriod(document) {
        var startDate = moment(document.selectedPeriod.startDate);
        var endDate = moment(document.selectedPeriod.endDate);

        switch (document.selectedPeriod.periodType) {
            case periodType.DAILY:
                startDate = startDate.startOf('day');
                endDate = endDate.endOf('day');

                break;

            case periodType.HOURLY:
                startDate.hours(6).minutes(0).seconds(0);
                endDate.hours(17).minutes(59).seconds(59);

                break;

            case periodType.HALF_DAILY:
                startDate.startOf('day');
                endDate.hours(11).minutes(59).seconds(59);

                if (document.selectedPeriod.halfDayType === halfDailyType.PM) {
                    startDate.add(12, 'hours');
                    endDate.add(12, 'hours');
                }

                break;
        }

        return {startDate: startDate, endDate: endDate};
    }

    window.onbeforeunload=null;
});
