VM$.configuration = (function() {

    var configurationLoaded = false;
    var configuration = null;
    var pendingCallbacks = new Array();
    var ajsParamsModule = null;

    VM$(document).ready(importRequiredModulesAndGetConfiguration);

    function importRequiredModulesAndGetConfiguration() {
        requirejs(['aui/params'], function (loadedAjsParamsModule) {
            ajsParamsModule = loadedAjsParamsModule;
            getConfigurationIfUserIsLogged();
        });
    }

    function getConfigurationIfUserIsLogged() {
        if (isUserLogged()) {
            getConfiguration();
        }
    }

    function isUserLogged() {
        return ajsParamsModule.loggedInUser;
    }

    function getConfiguration() {
        const ajaxParameters = {
            type: 'GET',
            url: AJS.contextPath() + '/rest/vacation-manager/1.0/configuration',
            contentType: 'application/json'
        };

        return VM$.ajax(ajaxParameters)
            .done(function (data) { handleConfigurationSuccess(data); });
    }

    function handleConfigurationSuccess(data) {
        configuration = data;
        configurationLoaded = true;
        for (var cb = 0; cb < pendingCallbacks.length; cb++) {
            var pendingCallback = pendingCallbacks[cb];
            getProperty(pendingCallback.property, pendingCallback.callback);
        }
    }

    function getProperty(property, callback) {
        if (configurationLoaded == false) {
            pendingCallbacks.push({
                property : property,
                callback : callback
            });
        } else {
            callback(configuration[property]);
        }
    }

    function getShowUserPickerVacationWarning(callback) {
        getProperty("showUserPickerWarning", callback);
    }

    function getAllowUsersToViewStatistics(callback) {
        getProperty('allowUsersToViewStatistics', callback);
    }

    function getAllowHrChangeViewStatistics(callback) {
        getProperty('allowHrChangeStatisticsVisible', callback);
    }

    function getFields(callback) {
        getProperty("fields", callback);
    }

    return {
        getFields: getFields,
        getShowUserPickerVacationWarning : getShowUserPickerVacationWarning,
        getAllowUsersToViewStatistics: getAllowUsersToViewStatistics,
        getAllowHrChangeViewStatistics: getAllowHrChangeViewStatistics
    };
})();
