function MinutesPickerField(container, issueContainerUtils) {

	var halfDayCheckbox = appendHalfDayPicker(issueContainerUtils.getEndTimeMinutesField());
	var lastValuesToHalfDayChangeCheck = {
		startTimeToCheck: 0,
		checkboxSelectedState: false
	};

    halfDayCheckbox.change(onHalfDayCheckboxChange);
    issueContainerUtils.getStartTimeField().change(setMinutesFieldValueToHalfDayBasingOnUserWorkdayMinutes);
    onSubmitEnableMinutesPickerSoItsValueIsAttachedToRequestBody();

	function appendHalfDayPicker(minutesPicker) {
        var labelText = AJS.I18n.getText('vacationmanager.vacation.customfield.minutespicker.halfday');
        var checkbox = AJS.$('<input type="checkbox" style="margin-left: 5px;" name="vmMinutesPickerHalfDay" />');
        var spinner = AJS.$('<span class="aui-icon aui-icon-wait" name="vmMinutesPickerSpinner" style="display: none; margin-left: 5px;" />');
        AJS.$('<label>')
            .append(checkbox)
            .append(labelText)
            .append(spinner)
            .insertAfter(minutesPicker);

        return checkbox;
	}
	
    function onHalfDayCheckboxChange() {
        setTimePeriodFieldDisabled(isHalfDayCheckboxChecked);
        setMinutesFieldValueToHalfDayBasingOnUserWorkdayMinutes();
        updateLastHalfDayValues();
    }

    function setMinutesFieldValueToHalfDayBasingOnUserWorkdayMinutes() {
        var startTimeString = issueContainerUtils.getStartTimeField().val();

        if (!isHalfDayCheckboxChecked() || !startTimeString || !doHalfDayValueNeedsToBeUpdated()) {
            return;
        }
    	updateLastHalfDayValues();

    	setHalfDaySpinnerVisibility(true);
    	var reporterName = issueContainerUtils.getReporterName();
        issueContainerUtils.getUserWorkdayMinutes(reporterName, startTimeString)
            .then(setMinutesFieldValueToHalfDay);
    }
	
	function setMinutesFieldValueToHalfDay(userWorkday) {
		var halfWorkMinutes = roundToNearestMinutesOption(userWorkday.workMinutes / 2);

        var endTimeMinutesField = issueContainerUtils.getEndTimeMinutesField();
		endTimeMinutesField.val(halfWorkMinutes);
		issueContainerUtils.getStartTimeField().change();
		setHalfDaySpinnerVisibility(false);
	}

	function roundToNearestMinutesOption(minutes) {
		var optionsMinutesStep = getOptionsMinutesStep();
		return Math.round(minutes / optionsMinutesStep) * optionsMinutesStep;
    }

    function getOptionsMinutesStep() {
        var endTimeMinutesField = issueContainerUtils.getEndTimeMinutesField();
        var firstOptionMinutes = endTimeMinutesField.find('option').first().val();
        var secondOptionMinutes = endTimeMinutesField.find('option').eq(1).val();

        return parseInt(secondOptionMinutes, 10) - parseInt(firstOptionMinutes, 10);
    }
    
    function isHalfDayCheckboxChecked() {
    	return halfDayCheckbox.prop('checked');
    }
    
    function doHalfDayValueNeedsToBeUpdated() {
    	var startTimeString = issueContainerUtils.getStartTimeField().val();
    	var halfDayChekboxChecked = isHalfDayCheckboxChecked();
    	
    	return lastValuesToHalfDayChangeCheck.startTimeToCheck !== startTimeString ||
    		lastValuesToHalfDayChangeCheck.checkboxSelectedState !== halfDayChekboxChecked;
    }
    
    function updateLastHalfDayValues() {
    	lastValuesToHalfDayChangeCheck.startTimeToCheck = issueContainerUtils.getStartTimeField().val();
    	lastValuesToHalfDayChangeCheck.checkboxSelectedState = isHalfDayCheckboxChecked();
    }
	
	function setTimePeriodFieldDisabled(disabled) {
		issueContainerUtils.getEndTimeMinutesField().prop('disabled', disabled);
	}

	function onSubmitEnableMinutesPickerSoItsValueIsAttachedToRequestBody() {
        issueContainerUtils.getForm().submit(enableMinutesPicker);
    }

	function enableMinutesPicker() {
        issueContainerUtils.getEndTimeMinutesField().prop('disabled', false);
    }
	
	function setHalfDaySpinnerVisibility(visibility) {
		container.find('[name=vmMinutesPickerSpinner]').toggle(visibility);
	}

}
