function PeriodTypeFields(container, issueContainerUtils) {

	var periodTypeCfId = container.find('[name=vmPeriodTypeCfId]').val();
	var periodTypeDailyOptionId = container.find('[name=vmPeriodTypeOptionDailyId]').val();
	var periodTypeHalfDailyOptionId = container.find('[name=vmPeriodTypeOptionHalfDailyId]').val();
	var periodTypeHourlyOptionId = container.find('[name=vmPeriodTypeOptionHourlyId]').val();
	
    var startDateCfId = container.find('[name=chart_startDateCfId]').val();
    var endDateCfId = container.find('[name=chart_endDateCfId]').val();
    var startDateTimeCfId = container.find('[name=chart_startDateTimeCfId]').val();
    var endTimeMinutesCfId = container.find('[name=chart_endTimeMinutesCfId]').val();
    var halfDayCfId = container.find('[name=chart_halfDayCfId]').val();

	var displayData = [
		{option: periodTypeDailyOptionId, fields: [startDateCfId, endDateCfId]},
		{option: periodTypeHalfDailyOptionId, fields: [startDateCfId, halfDayCfId]},
		{option: periodTypeHourlyOptionId, fields: [startDateTimeCfId, endTimeMinutesCfId]}
	];
	
	var validPeriodTypesForVacationType = {
		'DAILY': [periodTypeDailyOptionId],
		'DAILY_OR_HALF_DAILY': [periodTypeDailyOptionId, periodTypeHalfDailyOptionId],
		'DAILY_OR_HOURLY': [periodTypeDailyOptionId, periodTypeHourlyOptionId]
	};

	function getFieldGroupForCf(cfId) {
		var fieldGroupForCf = container.find('label[for=customfield_'+cfId+']').parent();
		if (fieldGroupForCf.length === 0) {
			//checking radio input
			fieldGroupForCf = container.find('[name=customfield_' + cfId + ']').parents('fieldset');
		}
		return fieldGroupForCf;
	}
	
	function getReqestTypeField() {
		return container.find('.radio[name=customfield_'+periodTypeCfId+']');
	}
	
	function getCheckedReqestType() {
		return container.find('.radio[name=customfield_'+periodTypeCfId+'][checked]').val();
	}
	
	function bindOnChangeListenerToPeriodTypeRadio() {
		getReqestTypeField().change(function(evt) {
			showFieldsForOption(evt.currentTarget.value);
		});
	}
	
	function bindOnChangeListenerToIssueTypeField() {
		issueContainerUtils.getIssueTypeField().change(hidePeriodTypeOptionsUnavailableForVacationDocumentPeriodType);
	}
	
	function hideHalfDailyDefaultOption() {
		container.find('input[name=customfield_'+halfDayCfId+'][value=-1]').parent().hide();
	}

	function showFieldsForOption(optionId) {
		displayData.map(function(data) {
			data.fields.map(function(field) {
				getFieldGroupForCf(field).hide();
			});
		});
		
		displayData.map(function(data) {
			data.fields.map(function(field) {
				if (optionId === data.option) {
                    getFieldGroupForCf(field).show();
                }
			});
		});
	}
	
	function setPeriodTypeById(periodTypeId) {
		container.find('input[name=customfield_'+periodTypeCfId+'][value='+periodTypeId+']').prop('checked', true).change();
	}
	
	function isSelectedPeriodTypeFieldValid() {
		var vacationDocumentPeriodType = issueContainerUtils.getVacationDocumentPeriodType();
		var validPeriodTypes = validPeriodTypesForVacationType[vacationDocumentPeriodType];
		if (!validPeriodTypes) {
			throw new Error('Unsupported vacation document period type \'' + vacationDocumentPeriodType + '\'');
		}
		
		for (var i=0; i<validPeriodTypes.length; i++) {
			if (validPeriodTypes[i] === getCheckedReqestType()) {
				return true;
			}
		}
		return false;
	}
	
	function selectDefaultPeriodTypeIfCurrentIsInvalid() {
		// TODO get rid of try-catch and prevent errors caused by missing custom fields VM-503
		try {
			if (!isSelectedPeriodTypeFieldValid()) {
				setPeriodTypeById(periodTypeDailyOptionId);
			}
		} catch(error) {
			console.error(error);
		}
	}
	
    function hidePeriodTypeOptionsUnavailableForVacationDocumentPeriodType() {
        var vacationDocumentPeriodType = issueContainerUtils.getVacationDocumentPeriodType();

        showAllPeriodTypeOptions();

        switch (vacationDocumentPeriodType) {
            case 'DAILY':
                hidePeriodTypeOptionsContainer();
                break;
            case 'DAILY_OR_HALF_DAILY':
                hideHourlyPeriodOption();
                break;
            case 'DAILY_OR_HOURLY':
                hideHalfDailyPeriodOption();
                break;
        }
    }

    function showAllPeriodTypeOptions() {
        issueContainerUtils.getAllPeriodTypeOptions().parent().show();
        issueContainerUtils.getPeriodTypeOptionsContainer().show();
    }

    function hidePeriodTypeOptionsContainer() {
        issueContainerUtils.getPeriodTypeOptionsContainer().hide();
    }

    function hideHourlyPeriodOption() {
        issueContainerUtils.getHourlyPeriodOption().parent().hide();
    }
    
    function hideHalfDailyPeriodOption() {
        issueContainerUtils.getHalfDailyPeriodOption().parent().hide();
    }
	
	function init() {
        removeDefaultOptionsFromRadioButtonCustomFields();
		showFieldsForOption(getCheckedReqestType());
		bindOnChangeListenerToPeriodTypeRadio();
		bindOnChangeListenerToIssueTypeField();
	    hidePeriodTypeOptionsUnavailableForVacationDocumentPeriodType();
		selectDefaultPeriodTypeIfCurrentIsInvalid();
	}

	function removeDefaultOptionsFromRadioButtonCustomFields() {
        issueContainerUtils.getHalfDailyDefaultOption().parent().remove();
        issueContainerUtils.getPeriodTypeDefaultOption().parent().remove();
	}
	
	init();
	
}
