var vmUtils = function() {
    var defaultFormattingOptions = {
        shortLabels: false      // whether to use short labels (e.g. 'd') or full-length labels (e.g. 'days')
    };

    return {
        formatDaysAndTime: formatDaysAndTime,
        formatDays: formatDays,
        formatTime: formatTime,
        trimDecimalPlacesIfInteger: trimDecimalPlacesIfInteger,
        getUrlQueryParams: getUrlQueryParams
    };

    function formatDaysAndTime(days, minutes, options) {
        options = options || defaultFormattingOptions;

    	var labelSeparator = ' ';
    	var daysLabel = AJS.I18n.getText('vacationmanager.text.days.lowercase');
    	var hoursLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.hours');
    	var minutesLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.minutes');

    	if (options.shortLabels) {
    		labelSeparator = '';
    		daysLabel = AJS.I18n.getText('vacationmanager.text.days.lowercase.short');
        	hoursLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.hours.short');
        	minutesLabel = AJS.I18n.getText('vacationmanager.vacation.customfield.statistics.minutes.short');
		}

    	if (days === 0 && minutes === 0) {
            return '0' + labelSeparator + daysLabel;
        }

        var dateSegments = [];
    	if (days) {
    		dateSegments.push(days + labelSeparator + daysLabel);
    	}

    	if (minutes) {
            var hours = Math.floor(minutes / 60);
            var remainingMinutes = minutes - hours * 60;
            if (hours > 0) {
            	dateSegments.push(hours + labelSeparator + hoursLabel);
            }
            
            if (remainingMinutes > 0) {
            	dateSegments.push(remainingMinutes + labelSeparator + minutesLabel);
            }
    	}
    	
    	return dateSegments.join(' ');
    }

    function formatDays(days, options) {
    	return formatDaysAndTime(days, 0, options);
    }

    function formatTime(minutes, options) {
    	return formatDaysAndTime(0, minutes, options);
    }

    function trimDecimalPlacesIfInteger(days) {
        if (isNotInteger(days)) {
            return days;
        }

        return Math.floor(days);
    }

    function isNotInteger(number) {
        return number % 1 !== 0;
    }
    
    function getUrlQueryParams() {
        var search = /([^&=]+)=?([^&]*)/g;
        var urlParams = {};
        var match;
        var urlQuery = window.location.search.substring(1);

        while (match = search.exec(urlQuery)) {
            urlParams[decode(match[1])] = decode(match[2]);
        }

        return urlParams;
    }

    function decode(text) {
        return decodeURIComponent(text.replace(/\+/g, ' '))
    }

}();